
/*  libcommon.so - library of commonly useful features for the Pyro
 *                 Operating System
 *
 *  Copyright (C) 2013 A. D. Sharpe <demetrious.sharpe(at)pyro-os.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

#ifndef __CONTAINER_H__
#define __CONTAINER_H__

#include <set>

#include <pyro/types.h>
#include <util/locker.h>
#include <common/debug.h>

#include <sys/types.h>


using namespace os;

namespace common
{

template<class T> class	Container
{
public:
	Container();
	~Container();
	
	status_t	Insert( T* pcObject );
	status_t	Remove( T* pcObject );
	
	int	GetTabCount( void )	const	{ FN_START();	return( m_nTabCount );	}
	int	GetMaxCount( void )	const	{ FN_START();	return( m_nMaxCount );	}
	int	GetAvgCount( void )	const	{ FN_START();	return( m_nAvgCount );	}
	
	bool HasObj( T* pcObject ) const;
	
	void MakeEmpty( void );
	bool IsEmpty( void ) const		{ FN_START();	return( m_tSet.empty() );}
	
	T*	GetFirstObj( void );
	T*	Begin( void );
	T*	GetLastObj( void );
	T*	End( void );
	T*	GetNextObj( T* pcObject );
	T*	GetPrevObj( T* pcObject );
	
private:
	void IncCount( void );
	void DecCount( void );
	
	std::set<T*>	m_tSet;
	Locker			*m_pcSetLock;
	
	int	m_nTabCount;
	int	m_nMaxCount;
	int	m_nAvgCount;
	
};

template<class T>
void Container<T>::IncCount()
{
	FN_START();
	
    m_nTabCount++;

    m_nAvgCount = (m_nAvgCount + m_nTabCount) / 2;

    if ( m_nTabCount > m_nMaxCount )
		m_nMaxCount = m_nTabCount;
	
	FN_END();
}

template<class T>
void Container<T>::DecCount()
{
	FN_START();
	
    m_nTabCount--;

    m_nAvgCount = (m_nAvgCount + m_nTabCount) / 2;
	
	FN_END();
}


template<class T>
Container<T>::Container()
{
	FN_START();
	
	m_pcSetLock = new Locker("__container__");
	
	MakeEmpty();
	
	FN_END();
}

template<class T>
Container<T>::~Container()
{
	FN_START();
	
	MakeEmpty();
	
	delete m_pcSetLock;
	
	FN_END();
}

template<class T>
status_t Container<T>::Insert( T* pcObject )
{
	FN_START();
	
	status_t nRet = EOK;
	
	AutoLocker _lock_(m_pcSetLock);

	if(!pcObject)
		nRet = -EFAULT;
	else {
		if(m_tSet.find(pcObject) != m_tSet.end())
			nRet = -EPERM;
		else {
			m_tSet.insert(pcObject);
			IncCount();
			nRet = EOK;
		}
	}
	
	FN_END();
	
	return nRet;
}

template<class T>
status_t Container<T>::Remove( T* pcObject )
{
	FN_START();
	
	status_t nRet = EOK;
	
	AutoLocker _lock_(m_pcSetLock);

	if(!pcObject)
		nRet = -EFAULT;
	else {
		if(m_tSet.find(pcObject) == m_tSet.end())
			nRet = -EPERM;
		else {
			m_tSet.erase(pcObject);
			DecCount();
			nRet = EOK;
		}
	}
	
	FN_END();
	
	return nRet;
}

template<class T>
bool Container<T>::HasObj( T* pcObject ) const
{
	FN_START();
	
	bool bRet = true;
	
	AutoLocker _lock_(m_pcSetLock);

	if(m_tSet.find(pcObject) == m_tSet.end())
		bRet = false;
	
	FN_END();
	
	return bRet;
}

template<class T>
void Container<T>::MakeEmpty( void )
{
	FN_START();
	
	AutoLocker _lock_(m_pcSetLock);

    m_nTabCount	=	0;
    m_nMaxCount	=	0;
    m_nAvgCount	=	0;
	
    m_tSet.clear();
    
	FN_END();
}

template<class T>
T* Container<T>::GetFirstObj( void )
{
	FN_START();
	FN_END();
		
	return Begin();
}

template<class T>
T* Container<T>::Begin( void )
{
	FN_START();
	
	AutoLocker _lock_(m_pcSetLock);
	
	typename std::set<T*>::iterator it;
	
	it = m_tSet.begin();
	
	FN_END();
	
	return (*it);
}

template<class T>
T* Container<T>::GetLastObj( void )
{
	FN_START();
	
	AutoLocker _lock_(m_pcSetLock);

	typename std::set<T*>::iterator it;
	
	it = --m_tSet.end();
	
	FN_END();
	
	return (*it);
}

template<class T>
T* Container<T>::End( void )
{
	FN_START();
	
	AutoLocker _lock_(m_pcSetLock);

	typename std::set<T*>::iterator it;
	
	it = m_tSet.end();
	
	FN_END();
	
	return (*it);
}

template<class T>
T* Container<T>::GetNextObj( T* pcObject )
{
	FN_START();
	
	AutoLocker _lock_(m_pcSetLock);
	
	typename std::set<T*>::iterator it;
	
	if(pcObject == NULL) {
		FN_END();
		return GetFirstObj();
	} else {
		if((it = m_tSet.find(pcObject)) == m_tSet.end()) {
			FN_END();
			return NULL;
		}
	}
	
	it++;
	
	FN_END();
	
	return (*it);
}

template<class T>
T* Container<T>::GetPrevObj( T* pcObject )
{
	FN_START();
	
	AutoLocker _lock_(m_pcSetLock);
	
	typename std::set<T*>::iterator it;
	
	if(pcObject == NULL) {
		FN_END();
		return GetLastObj();
	} else {
		if((it = m_tSet.find(pcObject)) == m_tSet.end()) {
			FN_END();
			return NULL;
		}
	}
	
	it--;
	
	FN_END();
		
	return (*it);
}

} /* namespace common */

#endif /* __CONTAINER_H__ */
